#ifndef _C84_H_
#define _C84_H_		/* avoid multiple includes */

/*
 * c84.h -- PIC 16C84 assembler includes and defines,
 *		CSE510 Project, Fall 1995
 *
 * Copyright (C) 1995-6 James Cleverdon
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Address:
 *	James Cleverdon
 *	16465 S.W. Estuary Dr. # 101
 *	Beaverton, OR 97006-7954, USA
 *
 *	jamesc@sequent.com
 *
 *
 * $Log:	c84.h,v $
 * Revision 1.1  96/01/03  11:43:05  jamesc
 * Added short pointer sp to VAL union and n_line
 * field to NODE (for listing line reference).
 * 
 * Revision 1.0  1995/12/19 01:28:34  jamesc
 * "CSE510_project"
 *
 */

#ident "$Id: c84.h,v 1.1 96/01/03 11:43:05 jamesc Exp Locker: jamesc $"

/*
 * generic stuff
 */

#ifndef NEL
#define NEL(x)          (sizeof(x) / sizeof(*x))/* # of elements in array */
#endif

#ifndef MAX
#define MAX(a, b)	((a) > (b) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a, b)	((a) < (b) ? (a) : (b))
#endif

#ifndef NULL
#define NULL		(0)			/* just in case		*/
#endif

#ifndef TRUE
#define TRUE		(1)			/* just in case		*/
#define FALSE		(0)
#endif

#define NNULL		((NODEP)NULL)		/* casted NULL		*/

#define LINE_LEN	256			/* max. line length	*/
#define SYM_LEN 	32			/* max. symbol len + 1	*/

/* just in case the compiler doesn't come with ANSI typedefs */

#ifndef uchar
#define uchar		unsigned char
#define ushort		unsigned short
#define uint		unsigned int
#define ulong		unsigned long
#endif

/*
 * yacc defines
 */

#undef YYSTYPE
#define YYSTYPE		NODEP

#define T_F_B		116	/* internal argument type tokens */
#define T_F_D		117
#define T_LINE		118

/*
 * symbol defines
 */

#define OP_RETLW	0x3400			/* opcode's value	*/

#define BIT_VAL(reg, b)	((reg) | ((b) << 7))	/* file,bit value	*/
#define DEST_VAL(rg, d)	((rg) | ((d) << 7))	/* file,dest value	*/

	/* node/symbol flags */
#define F_IS_VAL	0x01			/* is const or value	*/
#define F_BIN_OP	0x02			/* is binary operator	*/
#define F_OP_CD		0x04			/* is opcode word	*/
#define F_UNI_OP	0x08			/* is unary operator	*/
#define F_IS_CONST	0x10			/* is a constant	*/
#define F_DONT_LIST	0x20			/* built-in constant	*/
#define F_UNDEF_LABEL	0x40			/* is undefined		*/

typedef union sym_val_u {
	struct node_s	*node;
	long		l;
} SVAL;

typedef struct symbol_s {
	struct symbol_s *s_next;
	uchar		s_type;
	uchar		s_flags;
	uchar		s_col;
	long		s_lineno;
	SVAL		s_val;			/* value or node list	*/
	char		s_name[1];		/* variable size	*/
} SYMBOL;

typedef union value_u {
	ulong		l;
	ushort	 	s[sizeof(long) / sizeof(short)];
	short	 	*sp;
	char		*str;
	struct node_s	*node;
} VAL;

typedef struct node_s {
	struct node_s	*n_left;		/* linked list / lvalue */
	struct node_s	*n_right;		/* node's rvalue	*/
	struct node_s	*n_parent;		/* parent node or symbol*/
	uchar		n_tok;	 		/* token type		*/
	uchar		n_type;	 		/* variable type	*/
	uchar		n_flags;		/* misc. flag bits	*/
	uchar		n_col;			/* token's input column */
	long		n_lineno;
	char		*n_file;
	struct node_s	*n_line;		/* source code line	*/
	SYMBOL		*n_sym;			/* original symbol ptr	*/
	VAL		n_extra;		/* all kinds of stuff	*/
	VAL		n_val;			/* value / node rvalue	*/
} NODE, *NODEP;

	/* for convenience.... */
#define n_next		n_left
#define n_addr		n_extra.s[0]
#define n_opcode	n_extra.s[1]

#endif /* _C84_H_ */
