
%{

/*
 * lex.l -- lexer for 16C84 assembler, CSE510 Project, Fall 1995
 *
 *	Build a caseless lexer using flex's -i flag.
 *
 * Copyright (C) 1995-6 James Cleverdon
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Address:
 *	James Cleverdon
 *	16465 S.W. Estuary Dr. # 101
 *	Beaverton, OR 97006-7954, USA
 *
 *	jamesc@sequent.com
 *
 * $Log:	lex.l,v $
 * Revision 1.1  96/01/03  11:53:23  jamesc
 * Added code to support rudimentary listings,
 * namely removed line_buf[] and instead alloc
 * a T_LINE NODE for each source line.  Added a
 * FIRST_LINE state and init code to force a \n
 * into the unget buffer and buffer the first
 * source line.
 * 
 * Revision 1.0  1995/12/19 01:28:34  jamesc
 * "CSE510_project"
 *
 */

#ident "$Id: lex.l,v 1.1 96/01/03 11:53:23 jamesc Exp Locker: jamesc $"

#undef YY_CHAR		/* some older flexes define YY_CHAR to be uchar */
#define YY_CHAR	char

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "c84.h"
#include "c84_ext.h"
#include "x.tab.h"

#define YY_USER_INIT		BEGIN(FIRST_LINE)

#define RET_TOK(tok)		return (new_tok(tok))
#define RET_CODE(tok, opcode)	return (new_code(tok, opcode));
#define RET_NULL(tok)		col += yyleng; yylval = NNULL; return (tok)


%}

	/* lex states */
%x	AT_NL
%x	ST
%x	FIRST_LINE

	/* identifier:  starts with alpha or '_', followed by alphanum/'_' */
ID	[_A-Z:?][_A-Z0-9:?]*

	/* right context for keywords:  not followed by identifier char  */
RC	/[^_A-Z0-9:?]

	/* white space */
W	[ \t]

%%

	/* numeric constants (a motley gang of formats) */

B'[01]+'		{ return (const_n(yytext + 2, 2)); }
%[01]+			{ return (const_n(yytext + 1, 2)); }
(O|Q)'[0-7]+'		{ return (const_n(yytext + 2, 8)); }
[0-9]+			{ return (const_n(yytext, 10)); }
0x[0-9A-F]+		{ return (const_n(yytext, 16)); }
H'[0-9A-F]+'		{ return (const_n(yytext + 2, 16)); }
'.'			{ return (char_const(yytext + 1)); }
'\\[0-7]+'		{ return (const_n(yytext + 2, 8)); }
'\\.'			{ return (esc_const(yytext + 2)); }
A'.'			{ return (char_const(yytext + 2)); }
A'\\[0-7]+'		{ return (const_n(yytext + 3, 8)); }
A'\\.'			{ return (esc_const(yytext + 3)); }

	/* string const:  bad string check in XXX_string_const() */

\"			{ BEGIN(ST); begin_string_const(); }
<ST>[^"\n]*\\\"[^"\n]*	{ more_string_const(); }
<ST>[^"\n]*\"?		{
		BEGIN(INITIAL);		/* end of string */
		return (end_string_const());
			}

	/* opcodes */

ADDLW/[^_A-Z0-9:?]	{ RET_CODE(T_ADDLW,	0x3e00); }
ADDWF/[^_A-Z0-9:?]	{ RET_CODE(T_ADDWF,	0x0700); }
ANDLW/[^_A-Z0-9:?]	{ RET_CODE(T_ANDLW,	0x3900); }
ANDWF/[^_A-Z0-9:?]	{ RET_CODE(T_ANDWF,	0x0500); }
BCF/[^_A-Z0-9:?]	{ RET_CODE(T_BCF,	0x1000); }
BSF/[^_A-Z0-9:?]	{ RET_CODE(T_BSF,	0x1400); }
BTFSC/[^_A-Z0-9:?]	{ RET_CODE(T_BTFSC,	0x1800); }
BTFSS/[^_A-Z0-9:?]	{ RET_CODE(T_BTFSS,	0x1c00); }
CALL/[^_A-Z0-9:?]	{ RET_CODE(T_CALL,	0x2000); }
CLRF/[^_A-Z0-9:?]	{ RET_CODE(T_CLRF,	0x0180); }
CLRW/[^_A-Z0-9:?]	{ RET_CODE(T_CLRW,	0x0100); }
CLRWDT/[^_A-Z0-9:?]	{ RET_CODE(T_CLRWDT,	0x0064); }
COMF/[^_A-Z0-9:?]	{ RET_CODE(T_COMF,	0x0900); }
DECF/[^_A-Z0-9:?]	{ RET_CODE(T_DECF,	0x0300); }
DECFSZ/[^_A-Z0-9:?]	{ RET_CODE(T_DECFSZ,	0x0b00); }
GOTO/[^_A-Z0-9:?]	{ RET_CODE(T_GOTO,	0x2800); }
INCF/[^_A-Z0-9:?]	{ RET_CODE(T_INCF,	0x0a00); }
INCFSZ/[^_A-Z0-9:?]	{ RET_CODE(T_INCFSZ,	0x0f00); }
IORLW/[^_A-Z0-9:?]	{ RET_CODE(T_IORLW,	0x3800); }
IORWF/[^_A-Z0-9:?]	{ RET_CODE(T_IORWF,	0x0400); }
MOVF/[^_A-Z0-9:?]	{ RET_CODE(T_MOVF,	0x0800); }
MOVLW/[^_A-Z0-9:?]	{ RET_CODE(T_MOVLW,	0x3000); }
MOVWF/[^_A-Z0-9:?]	{ RET_CODE(T_MOVWF,	0x0080); }
NOP/[^_A-Z0-9:?]	{ RET_CODE(T_NOP,	0x0000); }
OPTION/[^_A-Z0-9:?]	{ RET_CODE(T_OPTION,	0x0062); }
RETFIE/[^_A-Z0-9:?]	{ RET_CODE(T_RETFIE,	0x0009); }
RETLW/[^_A-Z0-9:?]	{ RET_CODE(T_RETLW,	0x3400); }
RETURN/[^_A-Z0-9:?]	{ RET_CODE(T_RETURN,	0x0008); }
RLF/[^_A-Z0-9:?]	{ RET_CODE(T_RLF,	0x0d00); }
RRF/[^_A-Z0-9:?]	{ RET_CODE(T_RRF,	0x0c00); }
SLEEP/[^_A-Z0-9:?]	{ RET_CODE(T_SLEEP,	0x0063); }
SUBLW/[^_A-Z0-9:?]	{ RET_CODE(T_SUBLW,	0x3c00); }
SUBWF/[^_A-Z0-9:?]	{ RET_CODE(T_SUBWF,	0x0200); }
SWAPF/[^_A-Z0-9:?]	{ RET_CODE(T_SWAPF,	0x0e00); }
TRIS/[^_A-Z0-9:?]	{ RET_CODE(T_TRIS,	0x0060); }
XORLW/[^_A-Z0-9:?]	{ RET_CODE(T_XORLW,	0x3a00); }
XORWF/[^_A-Z0-9:?]	{ RET_CODE(T_XORWF,	0x0600); }

	/* psuedo-ops and assembler directives */

BIT/[^_A-Z0-9:?]	{ RET_TOK(T_BIT); }
BYTE/[^_A-Z0-9:?]	{ RET_TOK(T_DBYTE); }
DATA/[^_A-Z0-9:?]	{ RET_TOK(T_RETLW); }
DB/[^_A-Z0-9:?]		{ RET_TOK(T_DBYTE); }
DORG/[^_A-Z0-9:?]	{ RET_TOK(T_DORG); }
DW/[^_A-Z0-9:?]		{ RET_TOK(T_DWORD); }
DWORD/[^_A-Z0-9:?]	{ RET_TOK(T_DWORD); }
EEB/[^_A-Z0-9:?]	{ RET_TOK(T_EBYTE); }
EEBYTE/[^_A-Z0-9:?]	{ RET_TOK(T_EBYTE); }
EEW/[^_A-Z0-9:?]	{ RET_TOK(T_EWORD); }
EEWORD/[^_A-Z0-9:?]	{ RET_TOK(T_EWORD); }
END/[^_A-Z0-9:?]	{ RET_TOK(T_END); }
EQU/[^_A-Z0-9:?]	{ RET_TOK(T_EQU); }
EORG/[^_A-Z0-9:?]	{ RET_TOK(T_EORG); }
FILL/[^_A-Z0-9:?]	{ RET_TOK(T_FILL); }
FUSE/[^_A-Z0-9:?]	{ RET_TOK(T_FUSE); }
ID/[^_A-Z0-9:?]		{ RET_TOK(T_ID); }
ORG/[^_A-Z0-9:?]	{ RET_TOK(T_ORG); }
PORG/[^_A-Z0-9:?]	{ RET_TOK(T_ORG); }
RES/[^_A-Z0-9:?]	{ RET_TOK(T_DBYTE); }
UNEQU/[^_A-Z0-9:?]	{ RET_TOK(T_UNEQU); }
WORD/[^_A-Z0-9:?]	{ RET_TOK(T_DWORD); }

	/* labels must be in the first column and may be followed by a ':' */

^{ID}:?			{
		yylval = label_scan(yytext);
		return (yylval->n_tok);
			}

	/* other identifiers can occur anywhere on the line */

{ID}			{
		yylval = label_scan(yytext);
		return (yylval->n_tok);
			}

	/* comments start with a ';' through end of line */

\;.*			{ /* eat comments */ col += yyleng; }

	/* math operators */

\(			{ RET_TOK('('); }
\)			{ RET_TOK(')'); }
\~			{ RET_TOK('~'); }
\!			{ RET_TOK('!'); }
\+			{ RET_TOK('+'); }
\-			{ RET_TOK('-'); }
\*			{ RET_TOK('*'); }
\/			{ RET_TOK('/'); }
\%			{ RET_TOK('%'); }
\&			{ RET_TOK('&'); }
\|			{ RET_TOK('|'); }
\^			{ RET_TOK('^'); }
\<\<			{ RET_TOK(T_LSHIFT); }
\>\>			{ RET_TOK(T_RSHIFT); }
MAX/[^_A-Z0-9:?]	{ RET_TOK(T_MAX); }
MIN/[^_A-Z0-9:?]	{ RET_TOK(T_MIN); }
HIGH/[^_A-Z0-9:?]	{ RET_TOK(T_HIGH); }
LOW/[^_A-Z0-9:?]	{ RET_TOK(T_LOW); }

	/* relational operators */

\!\=			{ RET_TOK(T_LOG_NE); }
\=\=			{ RET_TOK(T_LOG_EQ); }
\>			{ RET_TOK('>'); }
\>\=			{ RET_TOK(T_LOG_GE); }
\<			{ RET_TOK('<'); }
\<\=			{ RET_TOK(T_LOG_LE); }

	/* asst punctuation */

=			{ RET_TOK('='); }	/* assignment or equality */
,			{ RET_TOK(','); }
\$			{ RET_TOK('$'); }	/* current prog counter	*/

	/* format prefixes for debug, serin, serout */

\#			{ RET_TOK('#'); }

{W}+			{ /* eat white space */ col += yyleng; }

	/*
	 * Use trick from "Lex & Yacc" (pub: O'Reilly & Associates) to buffer
	 * up each line in line_buf for better error reporting.  I've made a
	 * few changes because I return \n as a token.
	 *
	 * Each rule must add yyleng to col after assigning col in NODE.
	 * Usually, new_tok() takes care of this.
	 */
\n			{
		BEGIN(AT_NL);
		++col;
		yyless(0);
		RET_NULL('\n');	/* feed yacc's look-ahead */
			}
<FIRST_LINE>.*		{
		unput('\n');	/* start with newline */
		BEGIN(AT_NL);	/* store the first line */
			}
<AT_NL>\n.*		{
		NODEP	np;

		BEGIN(INITIAL);
		col = 0;
		np = (NODEP) alloc(sizeof(NODE) + yyleng + 1, "line buf");
		np->n_val.str = (char *) &np[1];
		(void) strcpy(np->n_val.str, yytext + 1);
		np->n_left = NNULL;
		np->n_tok = T_LINE;
		np->n_lineno = ++line_num;
		np->n_file = this_file;
		if (!list_head) {
			list_head = list_tail = np;
		} else {
			list_tail->n_left = np;
			list_tail = np;
		}
		/* give back all but nl */
		yyless(1);
			}

	/* flag the unknown character */
.			{
		nerr(NNULL, "Unknown character:  (%s)\n",
			prtable_char(*yytext));
			}

%%

/*++++ end of lex.l ++++ */

